"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lngLatFactory_1 = require("idly-common/lib/geo/lngLatFactory");
const nodeFactory_1 = require("idly-common/lib/osm/entityFactory/nodeFactory");
const relationFactory_1 = require("idly-common/lib/osm/entityFactory/relationFactory");
const wayFactory_1 = require("idly-common/lib/osm/entityFactory/wayFactory");
const relationMemberFactory_1 = require("idly-common/lib/osm/relationMemberFactory");
/**
 * Converts the osm xml to an array of Entity,
 * This implementation is a port from iD.
 */
function iDParser(xml) {
    if (!xml || !xml.childNodes) {
        return [];
    }
    let root = xml.childNodes[2];
    if (!root) {
        root = xml.childNodes[0];
    }
    const children = root.childNodes;
    const entities = [];
    const len = children.length;
    for (let i = 0; i < len; i++) {
        const child = children[i];
        if (child.nodeName === 'node') {
            entities.push(nodeData(child));
        }
        else if (child.nodeName === 'relation') {
            entities.push(relationData(child));
        }
        else if (child.nodeName === 'way') {
            entities.push(wayData(child));
        }
    }
    return entities;
}
exports.iDParser = iDParser;
/**
 * @param attrs
 */
function getVisible(attrs) {
    const item = attrs.getNamedItem('visible');
    if (!item) {
        return true;
    }
    return item.value !== 'false';
}
function getMembers(obj) {
    const elems = obj.getElementsByTagName('member');
    const members = [];
    const len = elems.length;
    for (let i = 0; i < len; i++) {
        const attrs = elems[i].attributes;
        members.push(relationMemberFactory_1.relationMemberFactory({
            id: (attrs.getNamedItem('type').value[0] +
                attrs.getNamedItem('ref').value),
            ref: (attrs.getNamedItem('type').value[0] +
                attrs.getNamedItem('ref').value),
            role: attrs.getNamedItem('role').value,
            type: attrs.getNamedItem('type').value,
        }));
    }
    return members;
}
function getNodes(obj) {
    const elems = obj.getElementsByTagName('nd');
    const nodes = [];
    const len = elems.length;
    for (let i = 0; i < len; i++) {
        nodes.push('n' + elems[i].attributes.getNamedItem('ref').value);
    }
    return nodes;
}
function getTags(obj) {
    const elems = obj.getElementsByTagName('tag');
    const t = {};
    for (let i = 0, l = elems.length; i < l; i++) {
        const attrs = elems[i].attributes;
        t[attrs.getNamedItem('k').value] = attrs.getNamedItem('v').value;
    }
    return t;
}
function getLoc(attrs) {
    const attrLon = attrs.getNamedItem('lon');
    const attrLat = attrs.getNamedItem('lat');
    const lon = attrLon ? attrLon.value : '';
    const lat = attrLat ? attrLat.value : '';
    return lngLatFactory_1.lngLatFactory([parseFloat(lon), parseFloat(lat)]);
}
function nodeData(obj) {
    const attrs = obj.attributes;
    return nodeFactory_1.nodeFactory({
        attributes: {
            changeset: attrs.getNamedItem('changeset')
                ? attrs.getNamedItem('changeset').value
                : undefined,
            timestamp: attrs.getNamedItem('timestamp')
                ? attrs.getNamedItem('timestamp').value
                : undefined,
            uid: attrs.getNamedItem('uid')
                ? attrs.getNamedItem('uid').value
                : undefined,
            user: attrs.getNamedItem('user')
                ? attrs.getNamedItem('user').value
                : undefined,
            version: attrs.getNamedItem('version').value,
            visible: getVisible(attrs),
        },
        id: 'n' + attrs.getNamedItem('id').value,
        loc: getLoc(attrs),
        tags: getTags(obj),
    }, false);
}
function relationData(obj) {
    const attrs = obj.attributes;
    return relationFactory_1.relationFactory({
        attributes: {
            changeset: attrs.getNamedItem('changeset')
                ? attrs.getNamedItem('changeset').value
                : undefined,
            timestamp: attrs.getNamedItem('timestamp')
                ? attrs.getNamedItem('timestamp').value
                : undefined,
            uid: attrs.getNamedItem('uid')
                ? attrs.getNamedItem('uid').value
                : undefined,
            user: attrs.getNamedItem('user')
                ? attrs.getNamedItem('user').value
                : undefined,
            version: attrs.getNamedItem('version').value,
            visible: getVisible(attrs),
        },
        id: 'r' + attrs.getNamedItem('id').value,
        members: getMembers(obj),
        tags: getTags(obj),
    }, false);
}
function wayData(obj) {
    const attrs = obj.attributes;
    return wayFactory_1.wayFactory({
        attributes: {
            changeset: attrs.getNamedItem('changeset')
                ? attrs.getNamedItem('changeset').value
                : undefined,
            timestamp: attrs.getNamedItem('timestamp')
                ? attrs.getNamedItem('timestamp').value
                : undefined,
            uid: attrs.getNamedItem('uid')
                ? attrs.getNamedItem('uid').value
                : undefined,
            user: attrs.getNamedItem('user')
                ? attrs.getNamedItem('user').value
                : undefined,
            version: attrs.getNamedItem('version').value,
            visible: getVisible(attrs),
        },
        id: 'w' + attrs.getNamedItem('id').value,
        nodes: getNodes(obj),
        tags: getTags(obj),
    }, false);
}
//# sourceMappingURL=iDParser.js.map