"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const genReadlines = require("gen-readlines");
const process_1 = require("./process");
function* lineParser(path) {
    const fd = fs.openSync(path, 'r');
    const stats = fs.fstatSync(fd);
    const reader = genReadlines(fd, stats.size);
    for (const line of reader) {
        yield line.toString();
    }
    fs.closeSync(fd);
}
/**
 * The parser for nodejs. It directly reads a file
 * line by line. Reading a the file line by line enables
 * this to load any kind of huge file.
 */
function* nodejsParser(path, opts = {}) {
    opts = Object.assign({ prependEntityChar: true, useRef: false, freeze: false }, opts);
    const lastObj = {
        pointer: undefined,
    };
    for (const r of lineParser(path)) {
        const result = process_1.xmlLineToObj(r, lastObj, opts);
        if (result) {
            yield process_1.objToStandard(result, opts);
        }
    }
}
exports.nodejsParser = nodejsParser;
//# sourceMappingURL=nodejsParser.js.map