"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const nodeFactory_1 = require("idly-common/lib/osm/entityFactory/nodeFactory");
const relationFactory_1 = require("idly-common/lib/osm/entityFactory/relationFactory");
const wayFactory_1 = require("idly-common/lib/osm/entityFactory/wayFactory");
const helpers_1 = require("./helpers");
function xmlLineToObj(r, lastObj, opts) {
    const line = r.trim();
    if (line === '') {
        return;
    }
    if (line[0] !== '<' || line[line.length - 1] !== '>') {
        throw new Error('unknown osm bbox xml format, ' + line);
    }
    const attributesObj = helpers_1.parseAttributes(line);
    if (line.startsWith('<nd')) {
        lastObj.pointer.nodes.push('n' + attributesObj.ref);
        return;
    }
    if (line.startsWith('<tag')) {
        lastObj.pointer.tags[attributesObj.k] = helpers_1.unescape(attributesObj.v);
        return;
    }
    if (line.startsWith('<member')) {
        attributesObj.id = helpers_1.prependEntityChar(attributesObj.ref, attributesObj.type, opts.prependEntityChar);
        attributesObj.ref = attributesObj.id;
        lastObj.pointer.members.push(attributesObj);
        return;
    }
    if (line.startsWith('<node')) {
        const toReturn = lastObj.pointer;
        lastObj.pointer = { attributes: attributesObj, type: 'node', tags: {} };
        return toReturn;
    }
    if (line.startsWith('</node>')) {
        return;
    }
    if (line.startsWith('<way')) {
        const toReturn = lastObj.pointer;
        lastObj.pointer = {
            attributes: attributesObj,
            type: 'way',
            nodes: [],
            tags: {},
        };
        return toReturn;
    }
    if (line.startsWith('</way>')) {
        return;
    }
    if (line.startsWith('<relation')) {
        const toReturn = lastObj.pointer;
        lastObj.pointer = {
            attributes: attributesObj,
            type: 'relation',
            members: [],
            tags: {},
        };
        return toReturn;
    }
    if (line.startsWith('</relation>')) {
        return;
    }
    if (line.startsWith('<?xml')) {
        return;
    }
    if (line.startsWith('<bounds')) {
        return;
    }
    if (line.startsWith('<osm')) {
        return;
    }
    if (line.startsWith('</osm>')) {
        return lastObj.pointer;
    }
    throw new Error('Invalid osm xml, please use /api/0.6/map?bbox=left,bottom,right,top');
}
exports.xmlLineToObj = xmlLineToObj;
function objToStandard(r, opts) {
    r.attributes = helpers_1.parseVisibility(r.attributes);
    if (r.type === 'node') {
        return nodeFactory_1.nodeFactory({
            id: helpers_1.prependEntityChar(r.attributes.id, r.type, opts.prependEntityChar),
            attributes: r.attributes,
            loc: {
                lon: parseFloat(r.attributes.lon),
                lat: parseFloat(r.attributes.lat),
            },
            tags: r.tags,
        }, opts.freeze);
    }
    else if (r.type === 'way') {
        return wayFactory_1.wayFactory({
            id: helpers_1.prependEntityChar(r.attributes.id, r.type, opts.prependEntityChar),
            attributes: r.attributes,
            tags: r.tags,
            nodes: r.nodes,
        }, opts.freeze);
    }
    else if (r.type === 'relation') {
        return relationFactory_1.relationFactory({
            id: helpers_1.prependEntityChar(r.attributes.id, r.type, opts.prependEntityChar),
            attributes: r.attributes,
            tags: r.tags,
            members: r.members,
        }, opts.freeze);
    }
    else {
        throw new Error('Unknown type ' + r.type);
    }
}
exports.objToStandard = objToStandard;
//# sourceMappingURL=process.js.map